"""
Provide support for the OAuth dance when running on Google App Engine

"""


from jinja2 import Template as JinjaTemplate



#API_ENDPOINT = 'https://api.projectplace.com'
from google.appengine.api import users

from main import BaseHandler
class CredentialsDance(BaseHandler):
	"""
	Generate OAuth credentials for the user.
	"""

	def get(self, mode=''):

		import ppapiaccess
		import gae_db

		# Extract client/application/consumer credentials from config file. 
		from ppapiaccess.credentials import credentials_from_cfg
		creds = credentials_from_cfg(filepath='apiaccess.cfg')

		
		# Initiate the OAuth dance
		#client = oauth.ProjectplaceClient(consumer_key=creds['consumer_key'], consumer_secret=creds['consumer_secret'], callback_url='')
		
		# We have to get user specific credentials
		# OAuth endpoints given in the Projectplace API documentation

		from requests_oauthlib import OAuth1Session

		request_token_url = '%s/initiate'%ppapiaccess.API_ENDPOINT
		authorization_base_url = '%s/authorize'%ppapiaccess.API_ENDPOINT
		access_token_url = '%s/token'%ppapiaccess.API_ENDPOINT


	    

		if mode == "login_with_copy_paste":
			# User will be routed to the login/authenticate page provided by api.projectplace.com, after completing that process
			# the user will have to manually copy a value from the address bar to the input field in a form. Instructions are provided.
			
			template_copy_paste_page = """
			<html><body> 
			<form action="/credentials/verify" method="post">
			<input type="hidden" name="oauth_token" value="{{oauth_token}}">
			<div style="padding-left:25px;padding-top:25px;font:medium normal verdana;"><span style="font: normal large bold verdana;">Manual copy-paste-submit-form process to authenticate access to Projectplace service via Public API</span>
			<ol>
			<li>Click on <a href="{{authorization_url}}" target="_blank" title="Go to Projectplace authentication">this link</a> (opens in new window) to get an authentication page from Projectplace to connect with your user account </li>
			<li>Fill in your details and grant permission.</li>
			<li>The authentication page will reload. Copy URL from the address bar; should look something like this:<br />
			 https://api.projectplace.com/authorize?oauth_token=5e969eade0829a286d3f4057fca0de9c&oauth_verifier=276edce2bff09761542b8b477bb55eec</li>
			<li>Enter the URL here: <input type="text" name="url" size="200"></li>
			<li><input type="submit" value="Continue"></li>
			</ol>
			</form>
			<br />
			<i>Once you register an application at api.projectplace.com, you get your own consumer/application key & secret, you setup that application 
			with this callback url: "xxx://xxxx/credentials/verify".<br />
			Then there is no need for this manual copy-paste-submit-form process and you can use the <a href="/credentials/login_with_callback">automated process</a> instead.
			</div></body></html>'
			"""


			# 1. Fetch a request token
			oauth_session = OAuth1Session(client_key=creds['consumer_key'], 
											client_secret=creds['consumer_secret'])
			data = oauth_session.fetch_request_token(request_token_url)
			
			# 2. Save tokens to memcache for later use
			gae_db.save_temporary_auth_secret(service_name='PROJECTPLACE', 
										auth_token=data['oauth_token'], 
										auth_secret=data['oauth_token_secret'])


			# 3. Return page to user with link to Projectplace for authorization
			#    and form for manual copy&paste
			authorization_url = oauth_session.authorization_url(authorization_base_url)


			content = dict()
			content['oauth_token'] = data['oauth_token']
			content['authorization_url'] = authorization_url

			template = JinjaTemplate(template_copy_paste_page)
			self.response.write(template.render(content))
			
			return 


		elif mode == 'login_with_callback':
			# If developer has registered his application at api.projectplace.com then this mode can be used and will not require any 
			# copy+paste from end user. After authentication process at api.projectplace.com the ".../crdentials/verify" (callback) will be 
			# done automatiacally.
			
			# 1. Fetch a request token
			oauth_session = OAuth1Session(client_key=creds['consumer_key'], 
											client_secret=creds['consumer_secret'])
			data = oauth_session.fetch_request_token(request_token_url)
			
			# 2. Save tokens to memcache for later use
			gae_db.save_temporary_auth_secret(service_name='PROJECTPLACE', 
										auth_token=data['oauth_token'], 
										auth_secret=data['oauth_token_secret'])


			# 3. Return page to user with link to Projectplace for authorization
			#    and form for manual copy&paste
			authorization_url = oauth_session.authorization_url(authorization_base_url)


			#authorization_url = ''#client.get_authorization_url()	#https://api.projectplace.com/authorize?oauth_token=<value>
			

			redirecting_to_projectplace_page = """
			<html>
			<head>
			<META http-equiv="refresh" content="2;URL={{authorization_url}}">
			</head>
			<body>
			Redirecting to Projectplace.com for authentication... Redirecting in 2 seconds...<br />
			</body>
			</html>"""
			template = JinjaTemplate(redirecting_to_projectplace_page)
			self.response.write(template.render({'authorization_url':authorization_url}))

			
		
		elif mode == "verify":
			# After user has logged in using the login/authentication dialogue provided by api.projectplace.com, 
			# either automatically (in case developer has registered app and supplied a callback url) or via the 
			# manual copy-paste-submit-form process.
			# The key data is: oauth_token & oauth_verifier, these will be used to get user specific 
			# (semi-permanent) credentials.


			oauth_session = OAuth1Session(client_key=creds['consumer_key'], 
											client_secret=creds['consumer_secret'])

			# Example redirect-url / authorization_response:
			# https://api.projectplace.com/authorize?oauth_token=245e84ac23ecb0a72ea9b4997ea0db95&oauth_verifier=10a4d429f2a32e5a8a5768cb1a57a409
  			# https://pp-api-sdk.appspot.com/credentials/verify?oauth_token=f571401fb6021f275b141020000cc4b9&oauth_verifier=06bf3ed9d3ba38c9bada6b8c8d6625cf
			# If posted in form, then <'url'>, if a callback form api.rpojectplace.com then <request.url>
			url = self.request.get('url', self.request.url)
			process_token = oauth_session.parse_authorization_response(url)
			
			# Example process_token content: 
			# {u'oauth_verifier': u'10a4d429f2a32e5a8a5768cb1a57a409', u'oauth_token': u'245e84ac23ecb0a72ea9b4997ea0db95'}
			 
			## "oauth_token_secret" is not included in the redirect-url generated by projectplace.com
			## after the user has granted access. Extract from memcache and add it. 
			process_token['oauth_token_secret'] = gae_db.get_temporary_auth_secret('PROJECTPLACE',process_token['oauth_token'])
			oauth_session._populate_attributes(process_token)
			user_token = oauth_session.fetch_access_token(access_token_url)
			

			# Example user_token content:
			# {u'oauth_token_secret': u'cc46ee0feca9ef31c6c746b703c60864515b894e', u'oauth_token': u'78f8976f77284f1cb1c22424379198df'}
			
			#creds['service'] = 'PROJECTPLACE'
			creds['oauth_token'] = user_token['oauth_token']
			creds['oauth_token_secret'] = user_token['oauth_token_secret']

			creds = self.add_creds_to_session(service_name='PROJECTPLACE', values=creds)

			me = ppapiaccess.get_me(creds)

		
			credentials_generated_page = """<html><body>
			<H3>Congratulations! You now have access to your data from Projectplace.com.</h3>
			<p><i>Your identity at Projectplace.com is:<br /> {{name}} ({{email}})</i></p><br />
			<h4>Proceed by storing these credentials so you don't have to re-authenticate again:</h4>
			<ul>
			<li><a href="/credentials/save_creds_in_localstore">Save locally on your current device</a> (will be preserved even after restarting your browser)</li>
			<li><a href="/credentials/save_creds_with_google_account">Save using Google User account</a> (will be accessible from any browser and device)</li>
			<li><a href="/">I don't care about later usage, just keep it as a server side session</a> (will be deleted when you close your browser)</li>
			</ul>

			<br /><br /><br /><br /><br /><b><i>Developer utilities (credential details):<i></b><br />
			- user_token: "{{user_token}}"<br />
			- user_token_secret: "{{user_token_secret}}"<p /></i></h6>
			</body></html>
			"""
			data = dict()
			data['name']	= me.name
			data['email'] 	= me.email
			data['user_token']	= creds['oauth_token']
			data['user_token_secret']	= creds['oauth_token_secret']
			
			template = JinjaTemplate(credentials_generated_page)
			self.response.write(template.render(data))
			return 


		elif mode == "save_creds_with_google_account":
			# Credentials will be saved permanently in datastore using the google user as key

			user = users.get_current_user()

			if not user:
				self.redirect(users.create_login_url(self.request.uri))


			# Get existing from Session store
			session_cred = self.get_creds_from_session(service_name='PROJECTPLACE')
			

			# Get existing entry / crete new for Google App Engine Datastore (permanent)
			gae_cred = gae_db.get_service_credential('PROJECTPLACE', user)
			if not gae_cred:
				gae_cred = gae_db.create_service_credential('PROJECTPLACE', user)
			gae_cred.set_values(session_cred)
			gae_cred.put()
			
			
			saved_with_google_acount_page = """
			<html>
			<head>
			<META http-equiv="refresh" content="2;URL=http://{{host}}/">
			</head>
			<body>
			Congratulations to ubiquitos access! The credentials have been saved with your Google Account. Redirecting in 2 seconds...<br />
			</body>
			</html>"""
			template = JinjaTemplate(saved_with_google_acount_page)
			self.response.write(template.render({'host':self.request.host, 'debug':''}))
			return


		elif mode == 'save_creds_in_localstore':
			# Credentials will be saved in web broser storage

			template = """\
			<html>
			<head>

			<META http-equiv="refresh" content="2;URL=http://{{host}}/">

			<!-- http://www.w3schools.com/html/html5_webstorage.asp -->
			<script>
			/*
			*  Save/get credentials in local storage. Local storage is a [{},]
			*  'service' is the key/value used as identifier 
			*  {'oauth_token': '<val>', 'oauth_token_secret':'<val>',
			*  'consumer_key':'<val>', 'consumer_secret':'<val>'}  
			*/

			function set_creds(service_name, creds){

				creds['service'] = service_name;

				sso_safe = localStorage.getItem('sso_safe');
			    values_in_safe = JSON.parse(sso_safe);
				if (!sso_safe){ 
					values_in_safe = Array(creds);
				}
			    else{
			    	found = false;
			        for (var index=0; index < values_in_safe.length; index++){
			  			if (values_in_safe[index]['service'] == service_name){
			            	values_in_safe[index] = creds;
			                found = true;
			            }
					}
			        if (!found){
			        	values_in_safe.push(creds);
			        }
			    }
			    
			    values = JSON.stringify(values_in_safe);
				localStorage.setItem('sso_safe', values);
			    
			    return true
			}

			function get_creds(service_name){
				sso_safe = localStorage.getItem('sso_safe');
			    values_in_safe = JSON.parse(sso_safe);
				if (sso_safe){
			    	for (var index=0; index < values_in_safe.length; index++){
			  			if (values_in_safe[index]['service'] == service_name){
			            	return values_in_safe[index];
			            }
					}
			    }
			    return null;
			}

			function del_creds(service_name){
				sso_safe = localStorage.getItem('sso_safe');
			    values_in_safe = JSON.parse(sso_safe);
				if (sso_safe){
			    	for (var index=0; index < values_in_safe.length; index++){
			  			if (values_in_safe[index]['service'] == service_name){
			            	values_in_safe.splice(index, 1);
			                values = JSON.stringify(values_in_safe);
							localStorage.setItem('sso_safe', values);
			                return true;
			            }
					}
			    }
			    return false;
			}


			</script>
			
			
			</head>

			<body>
			
			<script>
			if(typeof(Storage) != "undefined"){
				// Yes! localStorage and sessionStorage support!
				var creds = {};
				creds['oauth_token'] = '{{ oauth_token }}';
				creds['oauth_token_secret'] = '{{ oauth_token_secret }}';
				creds['consumer_key'] = '{{ consumer_key }}';
				creds['consumer_secret'] = '{{ consumer_key }}';  
				set_creds('PROJECTPLACE', creds);
				document.write('You credentials has been succesfully stored. <br /> Redirecting in 2 seconds...');
			}
			else{
				document.write('Sorry! Your browser is not capable of storing values (supported in Internet Explorer 8+, Firefox, Opera, Chrome, and Safari).');
			}

			</script>
			</body>

			</html>
			 """
			
			# Get existing from Session store
			session_cred = self.get_creds_from_session(service_name='PROJECTPLACE')
			
			session_cred['host'] = self.request.host
			template = JinjaTemplate(template)
			self.response.write(template.render(session_cred))
			return
			
			#self.response.write(template%creds)
			#return


		elif mode=='clear_session':
			""" Remove credentials data from server side session"""

			self.session.clear()  # Currently we don't store anything else than creds, so clear it all.
			
			clear_session_page = """
			<html>
			<head>
			<META http-equiv="refresh" content="2;URL=http://{{host}}/">
			</head>
			<body>
			Server side session has been cleared from credentials. Redirecting in 2 seconds...
			</body>
			</html>"""
			template = JinjaTemplate(clear_session_page)
			self.response.write(template.render({'host':self.request.host}))
			#self.redirect('/')
			return


		elif mode == "clear_for_google_account":
			# Credentials will cleared from datastore 
			user = users.get_current_user()

			if not user:
				self.redirect(users.create_login_url(self.request.uri))


			# Get existing entry from Google App Engine Datastore (permanent)
			gae_cred = gae_db.get_service_credential('PROJECTPLACE', user)
			if gae_cred:
				gae_cred.clear()
				gae_cred.put()
				

			cleared_from_google_acount_page = """
			<html>
			<head>
			<META http-equiv="refresh" content="2;URL=http://{{host}}/">
			</head>
			<body>
			Congratulations to successfull cleanup! The credentials have been deleted from the permanent storage. Redirecting in 2 seconds...<br />
			</body>
			</html>"""
			template = JinjaTemplate(cleared_from_google_acount_page)
			self.response.write(template.render({'host':self.request.host, 'debug':''}))
			return



		else:
			self.response.write('<html><body>Oops something went wrong... :( Not familiar with mode "%s"</body></html>'%mode)
			return


	def post(self, mode=""):
		"""
		This will be used when manually copy pasting oauth_verifier (see mode == 'login')
		and data is posted using a manual form.
		"""
		if mode == 'verify':
			self.get(mode=mode)
		else:
			raise RuntimeError('POST method not configured for this ("%s") mode.'%mode)






