class GenericDataContainer(object):
    """
    Base class primarily providing data storage; provides a standard __init__() 
    method that can add any kind of data (provided in a dictionary) as attributes 
    on the object. Used e.g. in conjunction with extracting data from SQL DB.
    The "columns" (refers to database columns) attribute act as a filter so that 
    in a subclass you can limit what attributes can be present in class (e.g. 
    {'id;'','name':''} ). You can also create an "alias" for the attribute if you 
    e.g. prefer to have a standardized attribute naming (e.g. {'packagename':'name'}. 
    """
    
    columns  = {}    # Per default no filtering, all data in initdict will be inlcuded
    def __init__(self, initdict={}):
        """ Create attributes for the data in initdict """
        if initdict:
            for key in initdict.keys():
                key_stripped    = key.strip()
                if not self.columns or self.columns.has_key(key_stripped):
                    self.__dict__[key_stripped]  = initdict[key]
                if self.columns.has_key(key_stripped) and self.columns[key_stripped]:
                    # Create and additional attribute/alias containing this value
                    self.__dict__[self.columns[key_stripped]] = initdict[key]
    
    def __str__(self):
        """ Return basic info about self """
        name = ''
        if hasattr(self, 'name'):
            name = ' "%s"'%self.name
        id = ''
        if hasattr(self, 'id'):
            id = ' (%s)'%self.id

        return '%s object%s%s'%(self.__class__.__name__, name, id)
        
    
    def identifier_matches(self, identifier, exactmatchonly=True):
        """ Return true if parameter "identifier" matches self.name or self.id 
            This is useful when going though a list of objects to look for one 
            according to id/name """
        if identifier==self.id:   # not possible to do lower in an id as it's assumed to be a number
            return True
        elif exactmatchonly:
            return identifier == self.name
        elif not exactmatchonly:
            # Replace with RegExp
            if self.name.lower().find(identifier.lower()) != -1:
                return True
            else:
                return False
    
    def __eq__(self, other):
        """ Comparing two objects """
        if hasattr(self, 'id') and hasattr(other, 'id'):
            return self.id == other.id
        elif type(self) == type(other):
            # If comparing to of the same type 
            return self.__dict__ == other.__dict__
        else:
            # Can not be the same if they're not even of same type
            return False
             
