"""
Sample script to show how to work with the ppapiaccess library.
This script includes calls to two basic data fetching
functions that are available in the library: get_me() and
get_projects().
To create more such functions look at the definitions of get_me()
and get_project() and you should be able to get started quickly.
Documentation is available at http://api.projectplace.com. 

By C.Dalid, 2013-05-15
"""

def main(creds):
    """
    Authenticate with Projectplace.com API and present
    some basic info about the user.

    @param creds: Credentials
    @type creds: {...} (see ppapiaccess.connect for further info)
    """
    from ppapiaccess import connect, get_me, get_projects

    connect(creds)  # This creates a global connection
    
    print
    print "Info fetched via API regarding current user:"
    me = get_me()
    print " - Name:", me.name
    print " - Email:", me.email

    projects = get_projects()
    print " - Number of available projects:",len(projects)
    print ' - %d Members in first project ("%s"):'%(len(projects[0].members), projects[0].name)
    for member in projects[0].members.values():
        print '   * %s'%member.name
        
    
    

if __name__ == '__main__':

    
    ## TODO: Support provision of credentials via arguments?
    ##       (Individual credentials, path to conf file)

    # Making use of default configuration file that comes with the
    # package. Uses pp-api-sdk's client key and secret. 
    # You should create a new application (free of charge) and get
    # your own client key and secret since pp-api-sdk can be changed
    # at any time and thus disabling your integration.
    # You create a new application by using the "API Console"
    # https://projectplace-apis.appspot.com/?app=pp
    confpath = ''
    if not confpath:
        from ppapiaccess import default_confpath
        confpath = default_confpath

    # Extract credentials from config file. 
    from ppapiaccess.credentials import credentials_from_cfg
    creds = credentials_from_cfg(filepath=confpath)

    
    # Values for "oauth_token_secret" and "oauth_token" are not
    # available in the default configuration file (default_confpath).
    # These values refer to user specific credentials and must be 
    # obtained through an OAuth authentication process. 
    # If you haven't specified these in config file or locally here
    # in this file then the process will automatically trigger such
    # OAuth authentication process when you run this script.
    # Afterwards you can take the generated token & secret and either
    # 1) Put them here in the file to use by this script alone
    # 2) Put them in the config file to make use of them in all scripts
    # 3) Store them in some other fashion accoring to your own design

    #creds['oauth_token'] = ''
    #creds['oauth_token_secret'] = ''
    
    
    ## Call main()
    main(creds)

    import sys
    print 'Script (%s) finished.'%sys.argv[0]

