
#!/usr/bin/env python
#
# This is an sample Python 2.7 AppEngine application that shows how to
#   1) log in a user using the Projectplace OAuth API and
#   2) extract their projects.
#

__author__ = "Christopher Dalid" 


import webapp2

from google.appengine.api import users

# Add path to third party libraries that are not supported native in Google App Engine Python 2.7
# https://developers.google.com/appengine/docs/python/tools/libraries27
import sys
sys.path.insert(0,'lib')




"""
Support for storing data in server side session.
This makes it possible to avoid sending the credentials back and forth from the web client.
"""
from webapp2_extras import sessions
class BaseHandler(webapp2.RequestHandler):
	"""
	Custom base class that extends standard RequestHandler with a 
	session data storage mechanism.

	Thanks for helpful instructions: 
	http://www.essentialtech.co.nz/content/using_session_google_app_engine_and_python_27
	Technical doc:
	http://webapp-improved.appspot.com/api/webapp2_extras/sessions.html
	"""



	def dispatch(self):

		import gae_db	# Custom interface to Google Datastorage

		# -----------------------------------------------------------
		# Get a session store for this request.
		# -----------------------------------------------------------	 
		self.session_store = sessions.get_store(request=self.request)



		# -----------------------------------------------------------
		# Populate session with any available credentials
		# -----------------------------------------------------------	 
		user = users.get_current_user()

		for service_name in ['PROJECTPLACE', 'LINKEDIN']:
			
			if not self.get_creds_from_session(service_name=service_name):
				# We do not have any credentials in session store, see if any available elsewhere
				service_from_browser = self.request.get('service', '')
				if service_from_browser and service_from_browser==service_name:
					# Data has been posted from webbrowsers's localStorage
					creds = dict()
					creds['oauth_token'] 		= self.request.get('oauth_token')
					creds['oauth_token_secret'] = self.request.get('oauth_token_secret')
					creds['consumer_key']		= self.request.get('consumer_key')
					creds['consumer_secret']	= self.request.get('consumer_secret')
					self.add_creds_to_session(service_name=service_name, values=creds)

				elif user:
					## Look in GAE Datastore
					gae_cred = gae_db.get_service_credential(service_name=service_name, user=user)
					if gae_cred and gae_cred.oauth_token and gae_cred.oauth_token_secret:	# We have data in persistent storage
						
						temp = dict()
						temp['oauth_token'] 		= gae_cred.oauth_token
						temp['oauth_token_secret'] 	= gae_cred.oauth_token_secret
						temp['consumer_key'] 		= gae_cred.consumer_key
						temp['consumer_secret'] 	= gae_cred.consumer_secret
						self.add_creds_to_session(service_name=service_name, values=temp)



		# -----------------------------------------------------------
		# Dispatch (standard RequestHandler begaviour) 
		# -----------------------------------------------------------	 
			
		try:
		    # Dispatch the request.
		    webapp2.RequestHandler.dispatch(self)
		finally:
		    # Save all sessions.
		    self.session_store.save_sessions(self.response)

	@webapp2.cached_property
	def session(self):
	    # Returns a session using the default cookie key.
	    return self.session_store.get_session()



	def get_creds_from_session(self, service_name):
		"""
		@param service_name: Identifier of cloud service 
		@type service_name: <entry from gae_db.CLOUD_SERVICES>
		"""
		# Get existing from Session store
		if self.session.get('credentials'):
			for cred in self.session.get('credentials'):
				if cred['service'] == service_name:
					return cred
		return dict() 


	def add_creds_to_session(self, service_name, values):
		"""
		@param service_name: Identifier of cloud service 
		@type service_name: <entry from gae_db.CLOUD_SERVICES>
		@param values: Credentials data
		@type values: {...}
		"""

		data = dict()
		data['service'] = service_name
		for key in ['consumer_key', 'consumer_secret', 'oauth_token', 'oauth_token_secret']:
			data[key] = values[key]

		if not self.session.get('credentials'):
			self.session['credentials'] = list()
		else:
			for cred in self.session['credentials']:
				if cred['service'] == service_name:
					cred = data
					return

		self.session['credentials'].append(data)
		return self.get_creds_from_session(service_name=service_name)





"""
Support for making credentials available on all devices and browsers (permanent storage)
We support storing user credentials in AppEngine Datastorage for permanent access from all devices and browsers.
Implementation uses the unique identity property of a Google User Account (".user_id()") to keep track
of which credentials belong to who.
"""
from google.appengine.ext import ndb

def safe_combination(user):
    """
    Constructs a Datastore key for a Safe entity with user id.
    This key is used to group user specific credentials together.
    """
    return ndb.Key('Safe', user.user_id())


class GAECredential(ndb.Expando):
	""" Models individual OAuth credential """
	service = ndb.StringProperty() # What service is the credentials for
	token = ndb.StringProperty()
	secret = ndb.StringProperty()
	when_created = ndb.DateTimeProperty(auto_now_add=True) # When was credentials generated
	when_expires = ndb.DateTimeProperty()

	@classmethod
	def get_from_safe(cls, ancestor_key):
		return cls.query(ancestor=ancestor_key)





class Credential(object):
	"""
	Generic data store for credentials (not dependent on GAE).
	Objects of this kind are stored in session data.
	"""
	def __init__(self, service, token, secret=None, when_expires=None):
		self.service = service
		self.token = token
		self.secret = secret
		self.when_expires = when_expires 







"""
Request Handlers...
"""
from jinja2 import Template as JinjaTemplate
class MainHandler(BaseHandler):

 	def get(self, mode=""):

 		#import gae_db

		main_page = """<html>
		<head>

		<script>
			/*
			*  Save/get credentials in local storage. Local storage is a [{},]
			*  'service' is the key/value used as identifier 
			*  {'oauth_token': '<val>', 'oauth_token_secret':'<val>',
			*  'consumer_key':'<val>', 'consumer_secret':'<val>'}  
			*/

			function set_creds(service_name, creds){

				creds['service'] = service_name;

				sso_safe = localStorage.getItem('sso_safe');
			    values_in_safe = JSON.parse(sso_safe);
				if (!sso_safe){ 
					values_in_safe = Array(creds);
				}
			    else{
			    	found = false;
			        for (var index=0; index < values_in_safe.length; index++){
			  			if (values_in_safe[index]['service'] == service_name){
			            	values_in_safe[index] = creds;
			                found = true;
			            }
					}
			        if (!found){
			        	values_in_safe.push(creds);
			        }
			    }
			    
			    values = JSON.stringify(values_in_safe);
				localStorage.setItem('sso_safe', values);
			    
			    return true
			}

			function get_creds(service_name){
				sso_safe = localStorage.getItem('sso_safe');
			    values_in_safe = JSON.parse(sso_safe);
				if (sso_safe){
			    	for (var index=0; index < values_in_safe.length; index++){
			  			if (values_in_safe[index]['service'] == service_name){
			            	return values_in_safe[index];
			            }
					}
			    }
			    return null;
			}

			function del_creds(service_name){
				sso_safe = localStorage.getItem('sso_safe');
			    values_in_safe = JSON.parse(sso_safe);
				if (sso_safe){
			    	for (var index=0; index < values_in_safe.length; index++){
			  			if (values_in_safe[index]['service'] == service_name){
			            	values_in_safe.splice(index, 1);
			                values = JSON.stringify(values_in_safe);
							localStorage.setItem('sso_safe', values);
			                return true;
			            }
					}
			    }
			    return false;
			}


			</script>

		</head>


		<body>

		{% if not credentials %}
		<script>
		/* If we discover that needed credentials are stored on the client 
		   (from previous visits) then post them to server immediately */

		creds = get_creds('PROJECTPLACE');
		if (creds){
			// Create a form and post the data to server
			var form = document.createElement("form");
			form.action = window.location.href;
			form.method = "post"
			for(var key in creds) {
				input = document.createElement("input");
				input.type = 'hidden';
				input.name = key;
				input.value = creds[key];
  				form.appendChild(input);
			}
			document.body.appendChild(form);
			form.submit();

		}
		</script>
		{% endif %}



		<h2>Welcome to pp-api-sdk</h2>
		<img src="http://upload.wikimedia.org/wikipedia/commons/thumb/e/e4/Infobox_info_icon.svg/200px-Infobox_info_icon.svg.png" width="15">
		<div style="font: italic small normal verdana;">
		This is a web application that can speak with Projectplace.com. It's provided as a base for you as developer so 
		you can get started quickly without having to implement basic infrastructure such as handling of authentication (OAuth).<br />
		In addition to handling the authentication it also provides a sample on how to fetch basic data such as User and Projects 
		from Projectplace.com.
		</div>



		{% if not credentials %}
		<!-- SELECT CREDENTIALS (we do not have anything stored in the session since before) -->
		<div style="padding-left:25px;padding-top:30px;">
			<span style="font: normal large bold verdana;">Step 1</span> <span style="font: italic medium normal verdana;">(Setup this app to access your projects at Projectplace.)</span> 
			{% if not name_at_google %}
			UNLESS <span style="font: italic medium normal verdana;">you have previously saved <a href="{{ google_login }}">to your Google Account</a>....</span>
			{% endif %}
			<br />


			<a href="/credentials/login_with_copy_paste"><span style="font: underlined medium bold verdana;">Manual copy-paste-submit-form</span></a> or 
		<a href="/credentials/login_with_callback">Automated process</a> (<i>for automation  <a href="http://api.projectplace.com">register an application at api.projectplace.com</a> its free and easy)</i>
		<p>

		</div>

		<div style="padding-left:25px;padding-top:30px;">
			<span style="font: normal large bold verdana;">Step 2</span> <br />
			Go Crazy with your Projectplace stuff!
		</div>


		{% endif %}

		{% if credentials %}
		<a href="/projects">Some basic info about you</a><br />
		<a href="/people">Everyone in your Projectplace network!</a>


		{% endif %}

		<br /><br /><br /><br /><br /><b><i>Developer utilities:<i></b><br />
		<div style="font-size:small;"><i>
		{% if name_at_google %}
		You are logged in with Google User Account: {{name_at_google}}    (<a href="{{ google_logout }}">sign out</a>)
		{% else %}
		You are not logged in with any Google User Account.    (<a href="{{ google_login }}">sign in</a>)
		{% endif %}
		<br />
		<br />
		Credentials for Projectplace.com API (currently in session):<br />
		- user_token: "{{oauth_token}}"<br />
		- user_secret: "{{oauth_token_secret}}"<br />
		
		</i></div>
		<script>

		// DEBUGING:
		//values = JSON.stringify(localStorage);
		//alert('localStorage: '+values);
			

		creds = get_creds('PROJECTPLACE');
		if (creds){
			values = JSON.stringify(creds);
			document.write('<button onclick="del_creds(\\'PROJECTPLACE\\');alert(\\'Credentials removed from LocalStorage in browser\\');return false;"><i>Clear out my credentials from this device</i></button>');
		}
		</script>
		
		{% if credentials %}
		<button onclick="window.location.replace('/credentials/clear_session');"><i>Clear out my credentials from server side session</i></button>
		{% endif %}

		{% if cred_with_google %}
		<button onclick="window.location.replace('/credentials/clear_for_google_account');"><i>Clear out my credentials from permanent storage</i></button>
		{% endif %}
		<br />
		<br />
		<div style="font-size:small;"><i>
		Debug:<pre>{{debug}}</pre>
		</i></div>

		</body></html>
		"""
		template = JinjaTemplate(main_page)

		import gae_db

		data = dict()
		
		creds = self.get_creds_from_session(service_name='PROJECTPLACE')
		if creds:
			data = creds
		else:
			data['oauth_token'] = '' 
			data['oauth_token_secret'] = ''

		if data['oauth_token'] and data['oauth_token_secret']:
			data['credentials'] = True
		else:
			data['credentials'] = False


		data['name_at_google'] = ''
		data['google_logout'] = ''
		data['cred_with_google'] = False

		user = users.get_current_user()
		if user:
			data['name_at_google'] = user.nickname()
			data['google_logout'] = users.create_logout_url('/')
			gae_cred = gae_db.get_service_credential(service_name='PROJECTPLACE', user=user)
			if gae_cred and gae_cred.oauth_token and gae_cred.oauth_token_secret:	# We have data in persistent storage
				data['cred_with_google'] = True

		data['google_login'] = users.create_login_url('/')
		
		
		self.response.write(template.render(data))
		return


	def post(self, mode=""):
		"""
		This will be used when viewing data from Projectplace
		"""
		if mode == '':
			self.get(mode=mode)
		else:
			raise RuntimeError('POST method not configured for this ("%s") mode.'%mode)




def LinkedInHandler(BaseHandler):
	"""
	Do some LinkedIn stuff :)
	"""

	def get(self):

		self.response.write('Hello LinkedIn!')

		'''
		token = {u'access_token': u'AQWNmWX7LCyUqPr0ebUk4wIFb2_zZMNETe_kxpAD5qbU6XPTNU-an2aeBH41QXr56u8LTzw9a713m_ARJr_5Zx9cxJhZMgsMYkRvuystearoYGTeWllU9WbfyG0Z5V1D7fSbdP89j8tqhas3x1EMmhGxSOhb2Z0LJfx3yKkG440oyhGuScg', u'expires_in': 5183999}
		
		auth = get_authentication(token)
		
		application = linkedin.LinkedInApplication(auth)

		# Get myself / user logged in
		data = application.get_profile(selectors=['id', 'first-name', 'last-name', 'formatted-name', 'distance', 'picture-url', 'public-profile-url'])
		me = Person(id=data['id'], name=data['formattedName'], data=data)
		self.response.write(me)

		# Get some connections for the user
		data = application.get_connections(params={'start':10, 'count':1}, 
			selectors=['id', 'first-name', 'last-name', 'formatted-name', 'distance', 'picture-url', 'public-profile-url'])
		for entry in data['values']:
			self.response.write('<p>'+entry)
		'''
		return 



	
## Requirement for webappp2_extras.sessions (used in the custom BaseHandler class)
config = {}
config['webapp2_extras.sessions'] = {
    'secret_key': 'my-super-secret-key',
    'cookie_args':{'httponly':True}
}

import gae_projectplace_oauth
import projects
import people_projectplace


application = webapp2.WSGIApplication([
	('/credentials/(.*)', gae_projectplace_oauth.CredentialsDance),
	('/projects', projects.mainview),
	('/people', people_projectplace.main),
	('/(.*)', MainHandler),
	], 
	debug=True,
	config=config)





