"""
Model database structures

Sources of inspiration (thanks!):
- Python appengine_oauth_library

"""






"""
Support for making credentials available on all devices and browsers (permanent storage)
We support storing user credentials in AppEngine Datastorage for permanent access from all devices and browsers.
Implementation uses the unique identity property of a Google User Account (".user_id()") to keep track
of which credentials belong to who.
"""
from google.appengine.ext import ndb		# Permanent storage
from google.appengine.api import memcache   # Temporary storage



def _gae_sso_safe_key(user):
    """
    Constructs a Google App Engine Datastore Key to a Credentials Safe.
    Key is based on the user id of a Google Account. 
    This key is used to store generated cloud service credentials using
    Google Account as the identifier. 
    """
    return ndb.Key('Safe', user.user_id())

def _linkedin_sso_safe_key(user):
	"""
	This would be used if a LinkedIn Single Sign On mechanism is used,
	in that case the user id of a LinkedIn account is used as 
	base.
	"""
	raise RuntimeError('Not yet implemented, requires LinkedIn SSO support.')




CLOUD_SERVICES = dict()
CLOUD_SERVICES['pp'] = 'PROJECTPLACE'



class ServiceCredentials(ndb.Expando):
	""" Models individual OAuth credential """
	service = ndb.StringProperty(required=True) 			# What cloud service this (oauth) credentials is for
	client_token = ndb.StringProperty(required=True)		# Client/Application/Consumer token used when getting token&secret
	token = ndb.StringProperty()			# User specific token
	secret = ndb.StringProperty()			# User specific secret
	when_created = ndb.DateTimeProperty(auto_now_add=True) 	# When was credentials generated
	when_expires = ndb.DateTimeProperty()

	@classmethod
	def get_from_safe(cls, ancestor_key):
		return cls.query(ancestor=ancestor_key)




def _gen_memcache_auth_key(service_name, auth_token):
	return "oauth_%s_%s" % (service_name, auth_token)

def _save_temporary_auth_secret(service_name, auth_token, auth_secret):
	"""
	Saves the temporary auth token and secret (done before redirecting
		user to could service for login/granting of access to service)
	"""
	memcache.set(_gen_memcache_auth_key(auth_token), auth_secret, time=20*60)

def _get_temporary_auth_secret(service_name, auth_token):
	"""
	Get temporary auth secret
	"""
	key = _gen_memcache_auth_key(service_name=service_name, auth_token=auth_token)
	return memcache.get(key)






import webapp2
from jinja2 import Template as JinjaTemplate

class MainHandler(webapp2.RequestHandler):

	template_get = """
	<html>
	<body>
	{{ content }}<br />
	<br />
	Key: <pre>{{ key }}</pre>
	Value: <pre>{{ value }}</pre> 
	</body>
	</html>"""

	def get(self, mode):
		data = dict()
		data['content'] = 'Hello World!'
		data['key'] = self.request.get('key','<nothing>')

		value = self.request.get('value','')
		if value: 
			# Save
			_save_temporary_auth_secret('PROJECTPLACE', data['key'], value)
			value = ''
		else:
			# Extract and return
			value = _get_temporary_auth_secret('PROJECTPLACE', data['key'])
		data['value'] = value


		template = JinjaTemplate(self.template_get)
		self.response.write(template.render(data))










